@@ -10,7 +10,7 @@ __PACKAGE__->load_components(qw/PK::Auto/);
 sub last_insert_id {
   my $self = shift;
   $self->get_autoinc_seq unless $self->{_autoinc_seq};
-  $self->storage->dbh->last_insert_id(undef,undef,undef,undef,
+  $self->result_source->storage->dbh->last_insert_id(undef,undef,undef,undef,
     {sequence=>$self->{_autoinc_seq}});
 }
 
@@ -22,8 +22,8 @@ sub get_autoinc_seq {
     return $self->{_autoinc_seq} = $self->sequence;
   }
   
-  my @pri = keys %{ $self->_primaries };
-  my $dbh = $self->storage->dbh;
+  my @pri = $self->primary_columns;
+  my $dbh = $self->result_source->storage->dbh;
   while (my $col = shift @pri) {
     my $info = $dbh->column_info(undef,undef,$self->table,$col)->fetchrow_arrayref;
     if (defined $info->[12] and $info->[12] =~ 
