@@ -5,6 +5,10 @@ use warnings;
 
 use base qw/DBIx::Class/;
 
+__PACKAGE__->load_components(qw/AccessorGroup/);
+
+__PACKAGE__->mk_group_accessors('simple' => 'result_source');
+
 =head1 NAME 
 
 DBIx::Class::Row - Basic row methods
@@ -45,16 +49,20 @@ sub new {
   $obj->insert;
 
 Inserts an object into the database if it isn't already in there. Returns
-the object itself.
+the object itself. Requires the object's result source to be set, or the
+class to have a result_source_instance method.
 
 =cut
 
 sub insert {
   my ($self) = @_;
   return $self if $self->in_storage;
+  $self->{result_source} ||= $self->result_source_instance
+    if $self->can('result_source_instance');
+  my $source = $self->{result_source};
+  die "No result_source set on this object; can't insert" unless $source;
   #use Data::Dumper; warn Dumper($self);
-  $self->result_source->storage->insert(
-    $self->_table_name, { $self->get_columns });
+  $source->storage->insert($source->from, { $self->get_columns });
   $self->in_storage(1);
   $self->{_dirty_columns} = {};
   return $self;
@@ -88,7 +96,7 @@ sub update {
   my ($self, $upd) = @_;
   $self->throw( "Not in database" ) unless $self->in_storage;
   my %to_update = $self->get_dirty_columns;
-  return -1 unless keys %to_update;
+  return $self unless keys %to_update;
   my $rows = $self->result_source->storage->update(
                $self->result_source->from, \%to_update, $self->ident_condition);
   if ($rows == 0) {
@@ -114,20 +122,18 @@ sub delete {
   my $self = shift;
   if (ref $self) {
     $self->throw( "Not in database" ) unless $self->in_storage;
-    #warn $self->_ident_cond.' '.join(', ', $self->_ident_values);
     $self->result_source->storage->delete(
       $self->result_source->from, $self->ident_condition);
     $self->in_storage(undef);
-    #$self->store_column($_ => undef) for $self->primary_columns;
-      # Should probably also arrange to trash PK if auto
-      # but if we do, post-delete cascade triggers fail :/
   } else {
+    die "Can't do class delete without a ResultSource instance"
+      unless $self->can('result_source_instance');
     my $attrs = { };
     if (@_ > 1 && ref $_[$#_] eq 'HASH') {
       $attrs = { %{ pop(@_) } };
     }
     my $query = (ref $_[0] eq 'HASH' ? $_[0] : {@_});
-    $self->storage->delete($self->_table_name, $query);
+    $self->result_source_instance->resultset->search(@_)->delete;
   }
   return $self;
 }
@@ -161,7 +167,7 @@ Does C<get_column>, for all column values at once.
 
 sub get_columns {
   my $self = shift;
-  return return %{$self->{_column_data}};
+  return %{$self->{_column_data}};
 }
 
 =head2 get_dirty_columns
@@ -210,6 +216,7 @@ sub set_columns {
   while (my ($col,$val) = each %$data) {
     $self->set_column($col,$val);
   }
+  return $self;
 }
 
 =head2 copy
@@ -220,6 +227,13 @@ Inserts a new row with the specified changes.
 
 =cut
 
+sub copy {
+  my ($self, $changes) = @_;
+  my $new = bless({ _column_data => { %{$self->{_column_data}}} }, ref $self);
+  $new->set_column($_ => $changes->{$_}) for keys %$changes;
+  return $new->insert;
+}
+
 =head2 store_column
 
   $obj->store_column($col => $val);
@@ -239,35 +253,38 @@ sub store_column {
 
 =head2 inflate_result
 
-  Class->inflate_result(\%me, \%prefetch?)
+  Class->inflate_result($result_source, \%me, \%prefetch?)
 
 Called by ResultSet to inflate a result from storage
 
 =cut
 
 sub inflate_result {
-  my ($class, $me, $prefetch) = @_;
+  my ($class, $source, $me, $prefetch) = @_;
   #use Data::Dumper; print Dumper(@_);
-  my $new = bless({ _column_data => $me, _in_storage => 1 },
-                    ref $class || $class);
+  my $new = bless({ result_source => $source,
+                    _column_data => $me,
+                    _in_storage => 1
+                  },
+                  ref $class || $class);
   my $schema;
   PRE: foreach my $pre (keys %{$prefetch||{}}) {
-    my $rel_obj = $class->relationship_info($pre);
-    die "Can't prefetch non-eistant relationship ${pre}" unless $rel_obj;
-    $schema ||= $new->result_source->schema;
-    my $pre_class = $schema->class($rel_obj->{class});
-    my $fetched = $pre_class->inflate_result(@{$prefetch->{$pre}});
+    my $pre_source = $source->related_source($pre);
+    die "Can't prefetch non-existant relationship ${pre}" unless $pre_source;
+    my $fetched = $pre_source->result_class->inflate_result(
+                    $pre_source, @{$prefetch->{$pre}});
+    my $accessor = $source->relationship_info($pre)->{attrs}{accessor};
     $class->throw("No accessor for prefetched $pre")
-      unless defined $rel_obj->{attrs}{accessor};
-    PRIMARY: foreach my $pri ($rel_obj->{class}->primary_columns) {
+      unless defined $accessor;
+    PRIMARY: foreach my $pri ($pre_source->primary_columns) {
       unless (defined $fetched->get_column($pri)) {
         undef $fetched;
         last PRIMARY;
       }
     }
-    if ($rel_obj->{attrs}{accessor} eq 'single') {
+    if ($accessor eq 'single') {
       $new->{_relationship_data}{$pre} = $fetched;
-    } elsif ($rel_obj->{attrs}{accessor} eq 'filter') {
+    } elsif ($accessor eq 'filter') {
       $new->{_inflated_column}{$pre} = $fetched;
     } else {
       $class->throw("Don't know how to store prefetched $pre");
@@ -276,13 +293,6 @@ sub inflate_result {
   return $new;
 }
 
-sub copy {
-  my ($self, $changes) = @_;
-  my $new = bless({ _column_data => { %{$self->{_column_data}}} }, ref $self);
-  $new->set_column($_ => $changes->{$_}) for keys %$changes;
-  return $new->insert;
-}
-
 =head2 insert_or_update
 
   $obj->insert_or_update
@@ -306,6 +316,21 @@ sub is_changed {
   return keys %{shift->{_dirty_columns} || {}};
 }
 
+=head2 result_source
+
+  Accessor to the ResultSource this object was created from
+
+=head2 register_column($column, $column_info)
+
+  Registers a column on the class and creates an accessor for it
+
+=cut
+
+sub register_column {
+  my ($class, $col, $info) = @_;
+  $class->mk_group_accessors('column' => $col);
+}
+
 1;
 
 =head1 AUTHORS
